@@ -3,7 +3,7 @@ import os
 import sys
 import requests
 from zipfile import ZipFile
-import urllib.request
+import urllib
 from tempfile import NamedTemporaryFile
 from io import BytesIO, StringIO
 import numpy as np
@@ -15,7 +15,7 @@ EPSILON = 1e-10  # tolerance or comparison functions
 
 
 def mkdirs(path):
-    '''
+    """
     Makes directories to save output.
 
     Args:
@@ -24,7 +24,7 @@ def mkdirs(path):
     Returns:
         None
 
-    '''
+    """
 
     try:
         os.makedirs(path)
@@ -34,7 +34,7 @@ def mkdirs(path):
 
 
 def pct_diff_func(simul, data):
-    '''
+    """
     Used to calculate the absolute percent difference between data
     moments and model moments.
 
@@ -47,7 +47,7 @@ def pct_diff_func(simul, data):
     Returns:
         output (array_like): percentage differences between model and
             data moments
-    '''
+    """
     if np.asarray(data).all() != 0:
         frac = (simul - data) / data
         output = np.abs(frac)
@@ -57,7 +57,7 @@ def pct_diff_func(simul, data):
 
 
 def convex_combo(var1, var2, nu):
-    '''
+    """
     Takes the convex combination of two variables, where nu is in [0,1].
 
     Args:
@@ -69,13 +69,13 @@ def convex_combo(var1, var2, nu):
         combo (array_like): same shape as var1, convex combination of
         var1 and var2
 
-    '''
+    """
     combo = nu * var1 + (1 - nu) * var2
     return combo
 
 
 def read_file(path, fname):
-    '''
+    """
     Read the contents of 'path'. If it does not exist, assume the file
     is installed in a .egg file, and adjust accordingly.
 
@@ -86,7 +86,7 @@ def read_file(path, fname):
     Returns:
         file contents (str)
 
-    '''
+    """
 
     if not os.path.exists(os.path.join(path, fname)):
         path_in_egg = os.path.join("ogcore", fname)
@@ -97,9 +97,10 @@ def read_file(path, fname):
         return open(os.path.join(path, fname))
 
 
-def pickle_file_compare(fname1, fname2, tol=1e-3, exceptions={},
-                        relative=False):
-    '''
+def pickle_file_compare(
+    fname1, fname2, tol=1e-3, exceptions={}, relative=False
+):
+    """
     Read two pickle files and unpickle each. We assume that each
     resulting object is a dictionary. The values of each dict are either
     numpy arrays or else types that are comparable with the == operator.
@@ -113,17 +114,24 @@ def pickle_file_compare(fname1, fname2, tol=1e-3, exceptions={},
 
     Returns:
         comparison (bool): whether therea two dictionaries are the same
-    '''
+    """
     pkl1 = safe_read_pickle(fname1)
     pkl2 = safe_read_pickle(fname2)
-    comparison = dict_compare(fname1, pkl1, fname2, pkl2, tol=tol,
-                              exceptions=exceptions, relative=relative)
+    comparison = dict_compare(
+        fname1,
+        pkl1,
+        fname2,
+        pkl2,
+        tol=tol,
+        exceptions=exceptions,
+        relative=relative,
+    )
 
     return comparison
 
 
 def comp_array(name, a, b, tol, unequal, exceptions={}, relative=False):
-    '''
+    """
     Compare two arrays in the L inifinity norm
     Return True if | a - b | < tol, False otherwise
     If not equal, add items to the unequal list
@@ -141,7 +149,7 @@ def comp_array(name, a, b, tol, unequal, exceptions={}, relative=False):
     Returns:
         (bool): whether two arrays are the same or not
 
-    '''
+    """
 
     if name in exceptions:
         tol = exceptions[name]
@@ -173,7 +181,7 @@ def comp_array(name, a, b, tol, unequal, exceptions={}, relative=False):
 
 
 def comp_scalar(name, a, b, tol, unequal, exceptions={}, relative=False):
-    '''
+    """
     Compare two scalars in the L inifinity norm
     Return True if abs(a - b) < tol, False otherwise
     If not equal, add items to the unequal list
@@ -190,7 +198,7 @@ def comp_scalar(name, a, b, tol, unequal, exceptions={}, relative=False):
     Returns:
         (bool): whether two arrays are the same or not
 
-    '''
+    """
 
     if name in exceptions:
         tol = exceptions[name]
@@ -212,9 +220,17 @@ def comp_scalar(name, a, b, tol, unequal, exceptions={}, relative=False):
         return True
 
 
-def dict_compare(name1, dict1, name2, dict2, tol, verbose=False,
-                 exceptions={}, relative=False):
-    r'''
+def dict_compare(
+    name1,
+    dict1,
+    name2,
+    dict2,
+    tol,
+    verbose=False,
+    exceptions={},
+    relative=False,
+):
+    r"""
     Compare two dictionaries. The values of each dict are either
     numpy arrays or else types that are comparable with the `==` operator.
     For arrays, they are considered the same if `|x - y| < tol` in
@@ -234,7 +250,7 @@ def dict_compare(name1, dict1, name2, dict2, tol, verbose=False,
     Returns:
         (bool): whether two dictionaries are the same or not
 
-    '''
+    """
 
     keys1 = set(dict1.keys())
     keys2 = set(dict2.keys())
@@ -263,19 +279,36 @@ def dict_compare(name1, dict1, name2, dict2, tol, verbose=False,
         unequal_items = []
         for k, v in dict1.items():
             if type(v) == np.ndarray:
-                check &= comp_array(k, v, dict2[k], tol, unequal_items,
-                                    exceptions=exceptions,
-                                    relative=relative)
+                check &= comp_array(
+                    k,
+                    v,
+                    dict2[k],
+                    tol,
+                    unequal_items,
+                    exceptions=exceptions,
+                    relative=relative,
+                )
             else:
                 try:
                     check &= comp_scalar(
-                        k, v, dict2[k], tol, unequal_items,
-                        exceptions=exceptions, relative=relative)
+                        k,
+                        v,
+                        dict2[k],
+                        tol,
+                        unequal_items,
+                        exceptions=exceptions,
+                        relative=relative,
+                    )
                 except TypeError:
                     check &= comp_array(
-                        k, np.array(v), np.array(dict2[k]), tol,
-                        unequal_items, exceptions=exceptions,
-                        relative=relative)
+                        k,
+                        np.array(v),
+                        np.array(dict2[k]),
+                        tol,
+                        unequal_items,
+                        exceptions=exceptions,
+                        relative=relative,
+                    )
 
         if verbose and unequal_items:
             frmt = "Name {0}"
@@ -287,7 +320,7 @@ def dict_compare(name1, dict1, name2, dict2, tol, verbose=False,
 
 
 def to_timepath_shape(some_array):
-    '''
+    """
     This function takes an vector of length T and tiles it to fill a
     Tx1x1 array for time path computations.
 
@@ -297,13 +330,13 @@ def to_timepath_shape(some_array):
     Returns:
         tp_array (Numpy  array): reshaped array
 
-    '''
+    """
     tp_array = some_array.reshape(some_array.shape[0], 1, 1)
     return tp_array
 
 
 def get_initial_path(x1, xT, p, shape):
-    r'''
+    r"""
     This function generates a path from point x1 to point xT such that
     that the path x is a linear or quadratic function of time t.
 
@@ -326,7 +359,7 @@ def get_initial_path(x1, xT, p, shape):
             2. `xT` is the value at time `t=T-1: xT = a*(T-1)^2 + b*(T-1) + c`
             3. the slope of the path at `t=T-1` is 0: 0 = 2*a*(T-1) + b`
 
-    '''
+    """
     if shape == "linear":
         xpath = np.linspace(x1, xT, p.T)
     elif shape == "ratio":
@@ -337,17 +370,19 @@ def get_initial_path(x1, xT, p, shape):
         cc = x1
         bb = 2 * (xT - x1) / (p.T - 1)
         aa = (x1 - xT) / ((p.T - 1) ** 2)
-        xpath = (aa * (np.arange(0, p.T).reshape(p.T, 1, 1) ** 2) +
-                 (bb * np.arange(0, p.T).reshape(p.T, 1, 1)) + cc)
-    ending_x_tail = np.tile(xT.reshape(1, p.S, p.J),
-                            (p.S, 1, 1))
+        xpath = (
+            aa * (np.arange(0, p.T).reshape(p.T, 1, 1) ** 2)
+            + (bb * np.arange(0, p.T).reshape(p.T, 1, 1))
+            + cc
+        )
+    ending_x_tail = np.tile(xT.reshape(1, p.S, p.J), (p.S, 1, 1))
     xpath_full = np.append(xpath, ending_x_tail, axis=0)
 
     return xpath_full
 
 
 def safe_read_pickle(file_path):
-    '''
+    """
     This function reads a pickle from Python 2 into Python 2 or Python 3
 
     Args:
@@ -356,17 +391,17 @@ def safe_read_pickle(file_path):
     Returns:
         obj (object): object saved in pickle file
 
-    '''
-    with open(file_path, 'rb') as f:
+    """
+    with open(file_path, "rb") as f:
         try:
-            obj = pickle.load(f, encoding='latin1')
+            obj = pickle.load(f, encoding="latin1")
         except TypeError:  # pragma no cover
             obj = pickle.load(f)  # pragma no cover
     return obj
 
 
 def rate_conversion(annual_rate, start_age, end_age, S):
-    '''
+    """
     This function converts annual rates to model period ratesself.
 
     Args:
@@ -378,13 +413,13 @@ def rate_conversion(annual_rate, start_age, end_age, S):
     Returns:
         rate (array_like): model period rates
 
-    '''
+    """
     rate = (1 + annual_rate) ** ((end_age - start_age) / S) - 1
     return rate
 
 
 def save_return_table(table_df, output_type, path, precision=2):
-    '''
+    """
     Function to save or return a table of data.
 
     Args:
@@ -398,46 +433,44 @@ def save_return_table(table_df, output_type, path, precision=2):
     Returns:
         table_df (Pandas DataFrame): table
 
-    '''
-    pd.options.display.float_format = (
-        '{:,.' + str(precision) + 'f}').format
+    """
+    pd.options.display.float_format = ("{:,." + str(precision) + "f}").format
     if path is None:
-        if output_type == 'tex':
-            tab_str = table_df.to_latex(index=False, na_rep='')
+        if output_type == "tex":
+            tab_str = table_df.to_latex(index=False, na_rep="")
             return tab_str
-        elif output_type == 'json':
+        elif output_type == "json":
             tab_str = table_df.to_json(double_precision=precision)
             return tab_str
-        elif output_type == 'html':
+        elif output_type == "html":
             tab_html = table_df.to_html(
                 classes="table table-striped table-hover"
-                ).replace('\n', '')
-            tab_html.replace('\n', '')
+            ).replace("\n", "")
+            tab_html.replace("\n", "")
             return tab_html
         else:
             return table_df
     else:
-        if output_type == 'tex':
-            table_df.to_latex(buf=path, index=False, na_rep='')
-        elif output_type == 'csv':
-            table_df.to_csv(path_or_buf=path, index=False, na_rep='')
-        elif output_type == 'json':
-            table_df.to_json(path_or_buf=path,
-                             double_precision=precision)
-        elif output_type == 'excel':
-            table_df.to_excel(excel_writer=path, index=False, na_rep='')
+        if output_type == "tex":
+            table_df.to_latex(buf=path, index=False, na_rep="")
+        elif output_type == "csv":
+            table_df.to_csv(path_or_buf=path, index=False, na_rep="")
+        elif output_type == "json":
+            table_df.to_json(path_or_buf=path, double_precision=precision)
+        elif output_type == "excel":
+            table_df.to_excel(excel_writer=path, index=False, na_rep="")
         else:
-            print('Please enter a valid output format')  # pragma no cover
-            assert(False)  # pragma no cover
+            print("Please enter a valid output format")  # pragma no cover
+            assert False  # pragma no cover
 
 
-class Inequality():
-    '''
+class Inequality:
+    """
     A class with methods to compute different measures of inequality.
-    '''
+    """
 
     def __init__(self, dist, pop_weights, ability_weights, S, J):
-        '''
+        """
         Args:
             dist (Numpy array): distribution of endogenous variables
                 over age and lifetime income group, size, SxJ
@@ -451,12 +484,13 @@ class Inequality():
         Returns:
             None
 
-        '''
+        """
         self.dist = dist
         self.pop_weights = pop_weights
         self.ability_weights = ability_weights
-        weights = (np.tile(pop_weights.reshape(S, 1), (1, J)) *
-                   ability_weights.reshape(1, J))
+        weights = np.tile(
+            pop_weights.reshape(S, 1), (1, J)
+        ) * ability_weights.reshape(1, J)
         flattened_dist = dist.flatten()
         flattened_weights = weights.flatten()
         idx = np.argsort(flattened_dist)
@@ -464,8 +498,8 @@ class Inequality():
         self.sort_weights = flattened_weights[idx]
         self.cum_weights = np.cumsum(self.sort_weights)
 
-    def gini(self, type='overall'):
-        '''
+    def gini(self, type="overall"):
+        """
         Compute the Gini coefficient
 
         Args:
@@ -473,33 +507,33 @@ class Inequality():
 
         Returns:
             gini_coeff (scalar): Gini coefficient
-        '''
-        if type == 'overall':
+        """
+        if type == "overall":
             p = np.cumsum(self.sort_weights)
             nu = np.cumsum(self.sort_dist * self.sort_weights)
-        elif type == 'age':
+        elif type == "age":
             flattened_dist = self.dist.sum(axis=1).flatten()
             flattened_weights = self.pop_weights.flatten()
             idx = np.argsort(flattened_dist)
             sort_dist = flattened_dist[idx]
-            sort_weights = flattened_weights[idx]/flattened_weights.sum()
+            sort_weights = flattened_weights[idx] / flattened_weights.sum()
             p = np.cumsum(sort_weights)
-            nu = np.cumsum(sort_dist*sort_weights)
-        elif type == 'ability':
+            nu = np.cumsum(sort_dist * sort_weights)
+        elif type == "ability":
             flattened_dist = self.dist.sum(axis=0).flatten()
             flattened_weights = self.ability_weights.flatten()
             idx = np.argsort(flattened_dist)
             sort_dist = flattened_dist[idx]
-            sort_weights = flattened_weights[idx]/flattened_weights.sum()
+            sort_weights = flattened_weights[idx] / flattened_weights.sum()
             p = np.cumsum(sort_weights)
-            nu = np.cumsum(sort_dist*sort_weights)
+            nu = np.cumsum(sort_dist * sort_weights)
         nu = nu / nu[-1]
         gini_coeff = (nu[1:] * p[:-1]).sum() - (nu[:-1] * p[1:]).sum()
 
         return gini_coeff
 
     def var_of_logs(self):
-        '''
+        """
         Compute the variance of logs
 
         Args:
@@ -508,18 +542,19 @@ class Inequality():
         Returns:
             var_ln_dist (scalar): variance of logs
 
-        '''
+        """
         ln_dist = np.log(self.sort_dist)
-        weight_mean = ((
-            ln_dist * self.sort_weights).sum() / self.sort_weights.sum())
-        var_ln_dist = ((
-            (self.sort_weights * ((ln_dist - weight_mean) ** 2)).sum())
-                       * (1. / (self.sort_weights.sum())))
+        weight_mean = (
+            ln_dist * self.sort_weights
+        ).sum() / self.sort_weights.sum()
+        var_ln_dist = (
+            (self.sort_weights * ((ln_dist - weight_mean) ** 2)).sum()
+        ) * (1.0 / (self.sort_weights.sum()))
 
         return var_ln_dist
 
     def ratio_pct1_pct2(self, pct1, pct2):
-        '''
+        """
         Compute the pct1/pct2 percentile ratio
 
         Args:
@@ -533,7 +568,7 @@ class Inequality():
 
         Notes:
             usually pct1 > pct 2
-        '''
+        """
         assert pct1 > 0
         assert pct1 < 1
         assert pct2 > 0
@@ -545,7 +580,7 @@ class Inequality():
         return pct_ratio
 
     def pct(self, pct):
-        '''
+        """
         Returns value at given percentile
 
         Args:
@@ -555,7 +590,7 @@ class Inequality():
         Returns:
             value (scalar): value of variable at pct
 
-        '''
+        """
         assert pct > 0
         assert pct < 1
         loc_pct = np.argmin(np.abs(self.cum_weights - pct))
@@ -564,7 +599,7 @@ class Inequality():
         return value
 
     def top_share(self, pctile):
-        '''
+        """
         Compute the top X% share
 
         Args:
@@ -574,21 +609,27 @@ class Inequality():
         Returns:
             pctile_share (scalar): share of variable attributed to the
                 top pctile group
-        '''
+        """
         assert pctile > 0
         assert pctile < 1
         loc_pctile = np.argmin(np.abs(self.cum_weights - (1 - pctile)))
-        pctile_share = ((
-            self.sort_dist[loc_pctile:] *
-            self.sort_weights[loc_pctile:]).sum() /
-                        (self.sort_dist * self.sort_weights).sum())
+        pctile_share = (
+            self.sort_dist[loc_pctile:] * self.sort_weights[loc_pctile:]
+        ).sum() / (self.sort_dist * self.sort_weights).sum()
 
         return pctile_share
 
 
-def print_progress(iteration, total, source_name='', prefix='Progress:',
-                   suffix='Complete', decimals=1, bar_length=50):
-    '''
+def print_progress(
+    iteration,
+    total,
+    source_name="",
+    prefix="Progress:",
+    suffix="Complete",
+    decimals=1,
+    bar_length=50,
+):
+    """
     Prints a progress bar to the terminal when completing small tasks
     of a larger job.
 
@@ -611,34 +652,34 @@ def print_progress(iteration, total, source_name='', prefix='Progress:',
         bar (string): progress bar
 
     Returns: status
-    '''
-    status = 'Incomplete'
+    """
+    status = "Incomplete"
     str_format = "{0:." + str(decimals) + "f}"
     percents = str_format.format(100 * (iteration / float(total)))
     filled_length = int(round(bar_length * iteration / float(total)))
-    bar = '█' * filled_length + '-' * (bar_length - filled_length)
+    bar = "█" * filled_length + "-" * (bar_length - filled_length)
 
     if iteration == 0:
-        if source_name == '':
-            sys.stdout.write('Accessing data files...\n')
+        if source_name == "":
+            sys.stdout.write("Accessing data files...\n")
         else:
-            sys.stdout.write('Accessing ' + source_name +
-                             ' data files...\n')
+            sys.stdout.write("Accessing " + source_name + " data files...\n")
 
-    sys.stdout.write('\r%s |%s| %s%s %s' %
-                     (prefix, bar, percents, '%', suffix)),
+    sys.stdout.write(
+        "\r%s |%s| %s%s %s" % (prefix, bar, percents, "%", suffix)
+    ),
 
     if iteration == total:
-        sys.stdout.write('\n')
-        sys.stdout.write('Computing...\n')
-        status = 'Complete'
+        sys.stdout.write("\n")
+        sys.stdout.write("Computing...\n")
+        status = "Complete"
     sys.stdout.flush()
 
     return status
 
 
 def fetch_files_from_web(file_urls):
-    '''
+    """
     Fetches zip files from respective web addresses and saves them as
     temporary files. Prints progress bar as it downloads the files.
 
@@ -663,16 +704,17 @@ def fetch_files_from_web(file_urls):
     Returns:
         local_paths (list of strings): local paths of temporary data
             files
-    '''
+    """
     local_paths = []
 
     iteration = 0
     total = len(file_urls)
-    _ = print_progress(iteration, total, source_name='SCF')
+    _ = print_progress(iteration, total, source_name="SCF")
 
     for file_url in file_urls:
-        # url = requests.get(file_url) (if using reuests package)
-        url = urllib.request.urlopen(file_url)
+        request = urllib.request.Request(file_url)
+        request.add_header("User-Agent", "Mozilla/5.0")
+        url = urllib.request.urlopen(request)
 
         f = NamedTemporaryFile(delete=False)
         path = f.name
@@ -689,13 +731,13 @@ def fetch_files_from_web(file_urls):
         f.close()
 
         iteration += 1
-        _ = print_progress(iteration, total, source_name='SCF')
+        _ = print_progress(iteration, total, source_name="SCF")
 
     return local_paths
 
 
-def not_connected(url='http://www.google.com/', timeout=5):
-    '''
+def not_connected(url="http://www.google.com/", timeout=5):
+    """
     Checks for internet connection status of machine.
 
     Args:
@@ -709,7 +751,7 @@ def not_connected(url='http://www.google.com/', timeout=5):
 
     Raises:
         ConnectionError: If no response from url withing timeout
-    '''
+    """
     try:
         _ = requests.get(url, timeout=timeout)
         return False
